// ---------------------------------------------------------------------
/**
 Copyright (c) 2004-2006 Honda Research Institute Japan, Co., Ltd.

 \file MICARY_PF_Struct.h
 \brief \~English Header file for Post Filter Struct
 \brief \~Japanese MICARY|XgtB^̋ʍ\̂̂߂̃wb_t@C(Post Filter)
 \author Masatoshi Tsutsumi
 \date 15 Nov 2006
 \version $Id: MICARY_PF_Struct.h 450 2009-03-16 10:12:38Z suzuki $
*/

// ---------------------------------------------------------------------
#ifndef	MICARY_PF_Struct_h_defined
#define	MICARY_PF_Struct_h_defined
// ---------------------------------------------------------------------
//#include <string>
#include "MICARYHA_struct.h"
#include "MICARY_struct.h"


// ---------------------------------------------------------------------

/**
 Gain Parameters struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	HA_Float	ha_fMinGain;	///\~Japanese ̍ŏQC
} MICARY_PrmGain;

/**
 Smoothing Priori SNR Parameters struct
 \author	Masatoshi Tsutsumi
 \date	29 Nov 2006
*/
typedef struct{
	HA_Float	ha_fB;	///\~Japanese ԕW(b)
	HA_Integer	ha_fBwFst;	///\~Japanese t[ςŋ`Kp鉺gr(bwFst)
	HA_Integer	ha_fBwFen;	///\~Japanese t[ςŋ`Kpgr(bwFen)
	HA_Integer	ha_fBwG;	///\~Japanese Lш敽ϕ(bwG)
	HA_Integer	ha_fBwL;	///\~Japanese ш敽ςŋ`(bwL)
} MICARY_PrmZeta;

/**
 Voice Power Parameters struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	HA_Float	ha_fB;	///\~Japanese ԕW(b)
	HA_Integer	ha_fBwFst;	///\~Japanese t[ςŋ`Kp鉺gr(bwFst)
	HA_Integer	ha_fBwFen;	///\~Japanese t[ςŋ`Kpgr(bwFen)
	HA_Integer	ha_fBwG;	///\~Japanese Lш敽ϕ(bwG)
	HA_Integer	ha_fBwL;	///\~Japanese ш敽ςŋ`(bwL)
	HA_Float	ha_fZminF;	///\~Japanese OSN̍ŏl (Frame)
	HA_Float	ha_fZmaxF;	///\~Japanese OSN̍ől (Frame)
	HA_Float	ha_fZminG;	///\~Japanese OSN̍ŏl (Global)
	HA_Float	ha_fZmaxG;	///\~Japanese OSN̍ől (Global)
	HA_Float	ha_fZminL;	///\~Japanese OSN̍ŏl (Local)
	HA_Float	ha_fZmaxL;	///\~Japanese OSN̍ől (Local)
	HA_Float	ha_fZthF;	///\~Japanese zetaF0̌W
	HA_Float	ha_fZpmin;	///\~Japanese zpeak̉l
	HA_Float	ha_fZpmax;	///\~Japanese zpeak̏l
	HA_Float	ha_fAmpL;	///\~Japanese ݊mvZLocal̑΂W
	HA_Float	ha_fAmpG;	///\~Japanese ݊mvZGlobal̑΂W
	HA_Float	ha_fAmpF;	///\~Japanese ݊mvZFramȇ΂W
	HA_Float	ha_fQmax;	///\~Japanese ݊m̏l
	HA_Float	ha_fQmin;	///\~Japanese ݊m̉l
} MICARY_PrmVoiceP;

/**
 Saved Voice Power Spectrum struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	MICARY_FloatSpecArray	micary_fsSmSNR;	///\~Japanese Ot[̕SN
	MICARY_FloatArray	micary_faSmSNRF;	///\~Japanese Ot[̕SN Frame
	MICARY_FloatArray	micary_faSmSNRP;	///\~Japanese Ot[̕SN Peak
} MICARY_SavedVoiceP;

/**
 Estimate S/N Parameters struct
 \author	Masatoshi Tsutsumi
 \date	23 Nov 2006
*/
typedef struct{
	HA_Float	ha_fAlp_pmin;	///\~Japanese ŏܗL(alp_pmin)
	HA_Float	ha_fAlp_pmag;	///\~Japanese ܗĽW(alp_pmag)
	HA_Float	ha_fA;	///\~Japanese OSN̍XV(a)
	HA_Float	ha_fXi_max;	///\~Japanese OSÑNbv(OP.xi_max)
	HA_Float	ha_fTheta_max;	///\~Japanese GH1߂ۂv̍ől(OP.theta_max)
	HA_Float	ha_fTheta_min;	///\~Japanese GH1߂ۂv̍ŏl(OP.theta_min)
} MICARY_PrmEstSN;

/**
 Saved Estimate S/N Values struct
 \author	Masatoshi Tsutsumi
 \date	23 Nov 2006
*/
typedef struct{
	MICARY_FloatSpecArray	micary_fsInputPSpec;	///\~Japanese ̓p[(Shatp)
	MICARY_FloatSpecArray	micary_fsSN;	///\~Japanese OSN(xi0)
} MICARY_SavedEstSN;

/**
 reverberation Noise Parameters struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	HA_Float	ha_fGamma;	///\~Japanese c萔(gamma)
	HA_Float	ha_fDelta;	///\~Japanese ډ(SRR)(delta)
} MICARY_PrmRevNoise;

/**
 Saved Reverberation Noise Values struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	MICARY_FloatSpecArray	micary_fsRevPSpec;	///\~Japanese Ot[̎cp[XyNg
} MICARY_SavedEstRev;

/**
 Leak Noise Parameters struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	HA_Float	ha_fAlp_leak;	///\~Japanese Leak parameter (used like 'eta')
} MICARY_PrmLeakNoise;

/**
 Initial noise of MCRA Parameters struct
 \author	Masatoshi Tsutsumi
 \date	13 Dec 2006
*/
typedef struct{
	HA_Float	ha_fDelta;	///\~Japanese m臒l
	HA_Integer	ha_iL;	///\~Japanese XṼt[
	HA_Float	ha_fAlp_s;	///\~Japanese ̓XyNg̕W
	HA_Float	ha_fNoise_compense;	///\~Japanese valin spetical parameter (1,1) [1?]
	HA_Float	ha_fAlp_dmin;	///~\Japanese Minimum speech probability (1,1) [0.05]
	HA_Integer	ha_iInit_frame;	///~\Japanese Initialize update frams (1,1) [100]
} MICARY_PrmInitMCRA;

/**
 Saved Initial noise of MCRA Values struct
 \author	Masatoshi Tsutsumi
 \date	13 Dec 2006
*/
typedef struct{
	MICARY_FloatSpecArray	micary_fsStateNoisePSpec;	///\~Japanese Gp[XyNg
	MICARY_FloatSpecArray	micary_fsSmin;	///\~Japanese ͐M̃p[XyNg̍ŏl
	MICARY_FloatSpecArray	micary_fsStmp;	///\~Japanese ͐M̃p[XyNg̈ꎞŏl
	MICARY_FloatSpecArray	micary_fsS;	///\~Japanese ͐M̕p[XyNg
	MICARY_FloatSpecArray	micary_fsSf;	///\~Japanese Frequency smoothed spectrum (N, M) [0?] for debug output
	HA_Num	ha_numProc;	///\~Japanese 
} MICARY_SavedInitMCRA;

/**
 MCRA Parameters struct
 \author	Masatoshi Tsutsumi
 \date	5 Dec 2006
*/
typedef struct{
	HA_Float	ha_fAlp_d;				///\~Japanese G̕W
	HA_Float	ha_fDelta;				///\~Japanese m臒l
	HA_Integer	ha_iL;					///\~Japanese XṼt[
	HA_Float	ha_fAlp_s;				///\~Japanese ̓XyNg̕W
	HA_Float	ha_fLeak_ampleak_factor;	///\~Japanese RGUW
	HA_Float	ha_fSs_floor;			///\~Japanese@FloorW
	HA_Float	ha_fNoise_compense;		///\~Japanese G␳l
	HA_Float	ha_fIn_Noise_compense;	///\~Japanese ͐M̒GBǉX2008.12.3
	HA_Integer	ha_iResetL;				///\~Japanese ύXɏ񐔂Zbg邩 1:ύXC0:ύXȂ ǉX2009.3.12
} MICARY_PrmMCRA;



typedef struct{///\~Japanese ǉiXj2008.11.14
	HA_Char		*ha_pcSavename;			///\~Japanese f[^ۑ 
	HA_Char		*ha_iInitFromfilename;	///\~Japanese lǂݏot@C̖O
	HA_Integer	ha_iInitFromfile_ID;	///\~Japanese lǂݏot@CID̏
	HA_Integer	ha_iFileSave;			///\~Japanese t@Cۑ邩ǂH1:ۑC0:ۑȂ
	HA_Integer  ha_iKeepPeriod;			///\~Japanese f[^ێt[
	
} MICARY_PrmSave;





typedef struct{
	HA_Integer iID;							//ID
	HA_Integer iSize;							//ۑl̔z̃TCY
	HA_Integer ha_iLowerFreq;					//g
	HA_Integer ha_iUpperFreq;					//g
	HA_Integer	ha_numCh;						//
    HA_Char cPad[100];						//pbh
}MICARY_SavedFileHeader;///\~Japanese ǉiXj2008.11.14


/**
 Saved MCRA Values struct
 \author	Masatoshi Tsutsumi
 \date	5 Dec 2006
*/
typedef struct{
	MICARY_FloatSpecArray	micary_fsStateNoisePSpec;	///\~Japanese Gp[XyNg
	MICARY_FloatSpecArray	micary_fsSmin;	///\~Japanese ͐M̃p[XyNg̍ŏl
	MICARY_FloatSpecArray	micary_fsStmp;	///\~Japanese ͐M̃p[XyNg̈ꎞŏl
	HA_Num	ha_numProc;	///\~Japanese 
	MICARY_FloatSpecArray	micary_fsS;	///\~Japanese ͐M̕p[XyNg
} MICARY_SavedMCRA;



/**
 Post Filter Parameters struct
 \author	Masatoshi Tsutsumi
 \date	27 Dec 2006
*/
typedef struct{
	MICARY_PrmMCRA sPrmMCRA;	///\~Japanese MCRAvZp[^
	HA_Float	ha_fAlp_d;	///\~Japanese G̕W
	HA_Float	ha_fDelta;	///\~Japanese m臒l
	HA_Integer	ha_iL;	///\~Japanese XṼt[
	HA_Float	ha_fAlp_s;	///\~Japanese ̓XyNg̕W
	HA_Float	ha_fLeak_ampleak_factor;	///\~Japanese
	HA_Float	ha_fSs_floor;	///\~Japanese
	HA_Float	ha_fNoise_compense;	///\~Japanese 
	
} MICARY_PrmPFExec;

/**
 Saved Post Filter Values struct
 \author	Masatoshi Tsutsumi
 \date	27 Dec 2006
*/
typedef struct{
	MICARY_SavedMCRA	*psSavedMCRA;	///\~Japanese GvZ̕ۑl
} MICARY_SavedPFExec;

/**
 Command Line Parameters struct
 \author	Masatoshi Tsutsumi
 \date	15 Nov 2006
*/
typedef struct{
	HA_String		cOriginalWavFile;	///<\~Japanese ̓t@C
	HA_Float		ha_fStartTime;	///<\~Japanes Jn[sec]
	HA_Float		ha_fEndTime;	///<\~Japanes I[sec]
	HA_String		cSeparateWavFile;	///<\~Japanese t@C
	HA_String		cOriginalSpecFile;	///<\~Japanese ̓XyNgt@C
	HA_Integer		ha_iStartFrame;	///<\~Japanese ͊Jnt[[frame]
	HA_Integer		ha_iEndFrame;	///<\~Japanese ͏It[[frame]
	HA_String		cSeparateSpecFile;	///<\~Japanese o̓XyNgt@C
	HA_String		cNoiseSpecFile;	///<\~Japanese GXyNgt@C
	HA_Integer		ha_iNFFT;			///<\~Japanese FFT[point]
	HA_Integer		ha_iWindow;			///<\~Japanese [sample]
	MICARY_WindowEnum	micary_nwnWindow;	///<\~Japanese 
	HA_Integer		ha_iShift;			///<\~Japanese Vtg[sample]
	HA_Float		ha_fUpperFreq;	///<\~Japanes ͏g[Hz]
	MICARY_PrmMCRA		micary_sPrmMCRA;	///<\~Japanes MCRAݒp[^
	MICARY_PrmLeakNoise		micary_sPrmLeakNoise;	///<\~Japanes Rݒp[^
	MICARY_PrmRevNoise		micary_sPrmRevNoise;	///<\~Japanes cp[^
	MICARY_PrmEstSN		micary_sPrmEstSN;	///<\~Japanes SNݒp[^
	MICARY_PrmVoiceP		micary_sPrmVoiceP;	///<\~Japanes ݊mݒp[^
	MICARY_PrmGain		micary_sPrmGain;	///<\~Japanes œKQCp[^
} MICARY_PFCmdArgs;
// ---------------------------------------------------------------------
#endif	// MICARY_PF_Struct_h_defined
